#!/bin/bash
#

# buildRoot.sh

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <c1|e16|e15|e14|e13|e10> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
  if [ -z ${basequal} ]; then
    echo "ERROR: please specify the qualifiers"
    usage
    exit 1
  fi
  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  for qual in "${qlist[@]}"; do
    case $qual in
      [ec][1-9]*)
        cqual=$qual
        ;;
      *)
        ssi_die "Unrecognized qualifier $qual"
    esac
  done

  if [[ -z "$cqual" ]]; then
    ssi_die "Base qualifier is mandatory"
  fi

  if [[ -z "${extraqual}" ]] && [[ -n "${bqual}" ]]; then
    ssi_die "build type is mandatory."
  fi
  basequal=${cqual}
}


set_compiler_flags()
{
  case ${cqual} in
    e1[0245])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=14
      ;;
    e16)
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      local std=17
      ;;
    e13)
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=14
      ;;
    c1)
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      local std=17
      ;;
    *)
      echo "ERROR: unrecognized compiler qualifier ${cqual}"
      usage
      exit 1
  esac
  export CC=${cc} CXX=${cxx} CXXFLAGS=-std=c++${std}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=jemalloc
origpkgver=v5_0_1
pkgver=${origpkgver}
ssibuildshims_version=v1_03_02
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgdotver=`echo ${pkgtarver} | sed -e 's/v//'`
mmdotver=`echo ${pkgdotver} | cut -d. -f1-2`
# Downloaded tarball.
pkgtardir=${package}-${pkgdotver}
pkgtarfile=${pkgtardir}.tar.bz2
extra_opts=()

get_this_dir
get_ssibuildshims
set_qualifiers

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
  debug) extra_opts+=(--enable-debug) ;;
  prof|opt) ;;
  * )
    echo "ERROR: please specify debug, opt, or prof"
    usage
    exit 1
;;
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
# remove existing sourcedir to avoid patch conflicts
rm -rf ${sourcedir}
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup package for build.
setup -B ${package} ${pkgver} -q +${fullqual//:/:+} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
 ssi_die "fake setup failed: unsupported qualifier?"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

makejarg=(-j ${ncore})

# Print active products.
ups active

cd ${sourcedir} && rm -rf "${pkgtardir}"
tar xf ${tardir}/${pkgtarfile} || ssi_die "Unable to untar source"

set_compiler_flags

set -x

cd "${sourcedir}/${pkgtardir}"

# Apply patches
patches=()
for patch in "${patches[@]}"; do
  [[ -r "${patch}" ]] || continue # Don't care if there isn't one this time.
  if grep -e '^\(---\|+++\) \([ab]\|root-[^/]*\)/' ${patch} >/dev/null 2>&1; then
    pval=1
  else
    pval=0
  fi
  patch ${DEBUG_PATCHES:+--verbose} -p${pval} < "${patch}" || ssi_die "application of patch ${patch} failed."
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd ${blddir}

if [ ${xcc} = "gcc" ]
then
echo "Building with gcc: $(which gcc) and g++: $(which g++)" 1>&2
elif [ ${xcc} = "clang" ]
then
echo "Building with clang: $(which clang) and clang++: $(which clang++)" 1>&2
fi

"${sourcedir}/${pkgtardir}/configure" \
  --prefix="$(dirname "${pkgdir}")" \
  --exec-prefix="${pkgdir}" \
  --with-jemalloc-prefix= "${extra_opts[@]}"

make "${makejarg[@]}" build_lib || ssi_die "${make} failed."

# Now install in the final place.
make "${makejarg[@]}" install_{bin,lib,include} || ssi_die "${make} install failed."

if ! [[ "${OS1}" == "Darwin" ]]; then
  make "${makejarg[@]}" build_doc install_doc
fi

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

set -x
pkgdirvarname="$(echo ${package} | tr '[a-z]' '[A-Z]')_DIR"
fqdirvarname="${pkgdirvarname%_DIR}_FQ_DIR"
fixem=("${pkgdir}/bin/${package}"{-config,.sh})

for file in "${fixem[@]}"; do
  perl -wapi\~ -e 's&\Q'"${pkgdir}"'\E&\${'"${fqdirvarname}"'}&; s&\Q'"$(dirname "${pkgdir}")"'\E&\${'"${pkgdirvarname}"'}&' \
    "${file}" || \
    ssi_die "Unable to fix paths in ${file}"
  rm -f "${file}~"
done

perl -wapi\~ -e 's&\Q'"${pkgdir}"'\E&\${pcfiledir}/../..&; s&\Q'"$(dirname "${pkgdir}")"'\E&\${pcfiledir}/../../..&' \
  ${pkgdir}/lib/pkgconfig/*.pc || ssi_die "Unable to fix paths in pkgconfig files."
rm -f ${pkgdir}/lib/pkgconfig/*.pc~
set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "setup failed"
echo "${package} is installed at $(eval echo \${${fqdirvarname}})"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
